@@ -52,7 +52,8 @@ option_app_id_cb (const gchar *option_name,
                     gpointer     data,
                     GError     **error)
 {
-  if (!g_application_id_is_valid (value)) {
+  if (!g_application_id_is_valid (value) ||
+      !g_dbus_is_name (value)) {
     g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE,
                  "\"%s\" is not a valid application ID", value);
     return FALSE;
@@ -132,18 +133,6 @@ init_server (int argc,
   if (g_getenv ("G_ENABLE_DIAGNOSTIC") == NULL)
     g_setenv ("G_ENABLE_DIAGNOSTIC", "0", TRUE);
 
-#ifdef HAVE_UBUNTU
-#ifndef ENABLE_DISTRO_PACKAGING
-  /* Set some env vars to disable the ubuntu modules. Their package will be 
-   * built using --enable-distro-packaging, but anyone running from git will
-   * get the right behaviour.
-   */
-  g_setenv ("LIBOVERLAY_SCROLLBAR", "0", TRUE);
-#endif
-  g_setenv ("UBUNTU_MENUPROXY", "0", TRUE);
-  g_setenv ("NO_UNITY_GTK_MODULE", "1", TRUE);
-#endif
-
   _terminal_debug_init ();
 
   /* Change directory to $HOME so we don't prevent unmounting, e.g. if the
